@@ -14,6 +14,8 @@
 
 """Pyrex extension for tracking loaded objects"""
 
+import sys
+
 from meliae import (
     _loader,
     warn,
@@ -22,7 +24,7 @@ from meliae import (
 
 
 class TestMemObjectCollection(tests.TestCase):
-    
+
     def test__init__(self):
         moc = _loader.MemObjectCollection()
         self.assertEqual(0, moc._active)
@@ -38,7 +40,6 @@ class TestMemObjectCollection(tests.TestCase):
         self.assertEqual(933, moc._test_lookup(933))
         self.assertEqual(933, moc._test_lookup(933+1024))
         self.assertEqual(933, moc._test_lookup(933L+1024L))
-        self.assertEqual(933, moc._test_lookup(933L+2**32-1))
 
     def test__len__(self):
         moc = _loader.MemObjectCollection()
@@ -448,3 +449,11 @@ class Test_MemObjectProxy(tests.TestCase):
         # to the type
         mop = self.moc.add(4, 'MyClass', 156, children=[2, 1, 8])
         self.assertEqual({1: 'a'}, mop.refs_as_dict())
+
+    def test_expand_refs_as_dict_unknown(self):
+        # We only expand objects that fall into known types, not just any value
+        mop_f = self.moc.add(1, 'frame', 300, value='func_foo')
+        self.moc.add(2, 'str', 25, value='a')
+        mop = self.moc.add(3, 'dict', 140, children=[2, 1])
+        as_dict = mop.refs_as_dict()
+        self.assertEqual({'a': mop_f}, as_dict)
